// *************************************
// ** ZXpandMouse.h                   **
// ** Class Definitions               **
// *************************************

enum JoyButton { UP, DOWN, LEFT , RIGHT, FIRE};

class MouseRptParser : public MouseReportParser
{
	protected:
		void OnMouseMove		(MOUSEINFO *mi);
		void OnLeftButtonUp		(MOUSEINFO *mi);
		void OnLeftButtonDown	(MOUSEINFO *mi);
		void OnRightButtonUp	(MOUSEINFO *mi);
		void OnRightButtonDown	(MOUSEINFO *mi);
		void OnMiddleButtonUp	(MOUSEINFO *mi);
		void OnMiddleButtonDown	(MOUSEINFO *mi);


	public:
		// Keep track of last mouse movement by direction
		// Up, Down, Left, Right, Fire
		unsigned long plMovedLastMills[5];

};

class AtariJoystick
{
	protected:
		// Joystick Output Time Controls
		const byte cbDebounce = 5; // Time before mouse responds. The hight this goes the longer button is depressed
		const byte cbDebounceLimit = 10; // Upper limit of mouse response
		const float cfAcceleration = 0.125;
		const float cfDecceleration = 0.2;

		byte bJoyPinsOut[FIRE+1]; // Up, Down, Left, Right, Fire
		float bJoyDebounce[FIRE] = {cbDebounce,cbDebounce,cbDebounce,cbDebounce};

		// Class functions
		void ButtonOff (byte bButton);

	public:
		void ButtonPress(byte bButton, unsigned long ulMills);
		AtariJoystick(byte bUp, byte bDown, byte bLeft, byte bRight, byte bFire);
};


// *********************
// ** Class Functions **
// *********************


// ** MouseRptParser **
	void MouseRptParser::OnMouseMove(MOUSEINFO *mi)
	{

		if(mi->dY != 0){
			if(mi->dY < -1.5){
				// Mouse UP
				// Get milliseconds since turn on. Used to comapre times for debounce
				plMovedLastMills[UP] = millis();
			} else if(mi->dY > 1.5){
				// Mouse DOWN
				// Get milliseconds since turn on. Used to comapre times for debounce
				plMovedLastMills[DOWN] = millis();
			}
		}

		if(mi->dX != 0){
			if(mi->dX < -1.5){
				// Mouse LEFT
				plMovedLastMills[LEFT] = millis();
			} else if(mi->dX > 1.5){
				// Mouse RIGHT
				plMovedLastMills[RIGHT] = millis();
			}
		}


	};
	// Simple button press
	void MouseRptParser::OnLeftButtonUp	(MOUSEINFO *mi)
	{
		plMovedLastMills[FIRE] = false;
	};
	void MouseRptParser::OnLeftButtonDown	(MOUSEINFO *mi)
	{
		plMovedLastMills[FIRE] = true;
	};
	void MouseRptParser::OnRightButtonUp	(MOUSEINFO *mi)
	{
		plMovedLastMills[FIRE] = false;
	};
	void MouseRptParser::OnRightButtonDown	(MOUSEINFO *mi)
	{
		plMovedLastMills[FIRE] = true;
	};
	void MouseRptParser::OnMiddleButtonUp	(MOUSEINFO *mi)
	{
		plMovedLastMills[FIRE] = false;
	};
	void MouseRptParser::OnMiddleButtonDown	(MOUSEINFO *mi)
	{
		plMovedLastMills[FIRE] = true;
	};

// ** AtariJoystick **

	// Contructor: With Pin Values;
	AtariJoystick::AtariJoystick(byte bUp, byte bDown, byte bLeft, byte bRight, byte bFire)
	{
		bJoyPinsOut[UP] = bUp;
		bJoyPinsOut[DOWN] = bDown;
		bJoyPinsOut[LEFT] = bLeft;
		bJoyPinsOut[RIGHT] = bRight;
		bJoyPinsOut[FIRE] = bFire;

		byte bCount;

		for(bCount = UP ; bCount < FIRE+1; bCount ++){
			pinMode(bJoyPinsOut[bCount], OUTPUT);
			digitalWrite(bJoyPinsOut[bCount], LOW);
		}
	};

	// Turn Off / Set Joystick axis LOW

	void AtariJoystick::ButtonOff (byte bButton){

		// Mouse Decceleration
		if (bJoyDebounce[bButton] - cfDecceleration > cbDebounce ){
			bJoyDebounce[bButton] -= cfDecceleration;
		} else {
			bJoyDebounce[bButton] = cbDebounce;
		}

		digitalWrite(bJoyPinsOut[bButton], LOW);
	}

	// Press Joystick: byte(button, up, down, left, right, fire);
	void AtariJoystick::ButtonPress(byte bButton, unsigned long ulMills)
	{
		if(bButton != FIRE){
			byte bOppositAxis = bButton+-1+(((bButton+1) % 2)*2);

			if(ulMills < bJoyDebounce[bButton]){
				// Mouse Acceleration
				if (bJoyDebounce[bButton] + cfAcceleration < cbDebounceLimit ) {
					// Higher than 22 freaks out the ZX81
					bJoyDebounce[bButton]+=cfAcceleration;
				} else {
					Serial.println(bJoyDebounce[bButton]);
				}
				// Turn of Oposite movement
				ButtonOff(bOppositAxis);

				// Turn on Movement
				digitalWrite(bJoyPinsOut[bButton], HIGH);

			} else if(ulMills > bJoyDebounce[bButton]) {
				ButtonOff(bButton);

			}
		} else {
			if(ulMills){
				digitalWrite(bJoyPinsOut[bButton], HIGH);
			} else {
				digitalWrite(bJoyPinsOut[bButton], LOW);
			}
		}
	};

